<?php

namespace Drupal\profile_manager\EventSubscriber;

use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\StorageTransformEvent;
use Drupal\profile_manager\OptionalModulesTrait;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Ensures optional module statuses are preserved during export/import.
 *
 * Optional modules allow site owners to customize their site according to
 * expectations set by the profile. Optional modules should never end up in
 * core.extension exports, and during the import process, they should not be
 * uninstalled. This class preserves their status on import, and excludes them
 * from the core.extension config during export.
 */
class HandleOptionalModulesCoreExtensions implements EventSubscriberInterface {

  use OptionalModulesTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::STORAGE_TRANSFORM_IMPORT][] = ['onImportTransform', 100];
    $events[ConfigEvents::STORAGE_TRANSFORM_EXPORT][] = ['onExportTransform', 100];
    return $events;
  }

  /**
   * Ensure that enabled optional modules remain enabled.
   */
  public function onImportTransform(StorageTransformEvent $event) {
    $storage = $event->getStorage();
    $extensions = $storage->read('core.extension');
    foreach ($this->getProfileOptionalModules() as $module) {
      $extensions['module'][$module] = 0;
    }
    $storage->write('core.extension', $extensions);
    // @todo use the profile's dependencies to generate a diff of enabled
    // modules that are dependencies of optional modules and ensure they remain
    // enabled.
  }

  /**
   * Do not export site specific optional module statuses.
   */
  public function onExportTransform(StorageTransformEvent $event) {
    $storage = $event->getStorage();
    $extensions = $storage->read('core.extension');
    foreach ($this->getProfileOptionalModules() as $module) {
      unset($extensions['module'][$module]);
    }
    $storage->write('core.extension', $extensions);
    // @todo compare core.extensions to the profile's dependencies to exclude
    // dependencies introduced by the optional modules.
  }

}
